package be.iminds.ilabt.jfed.highlevel.tasks;

import be.iminds.ilabt.jfed.fedmon.webapi.service.json.Server;
import be.iminds.ilabt.jfed.highlevel.controller.Task;
import be.iminds.ilabt.jfed.highlevel.controller.TaskExecution;
import be.iminds.ilabt.jfed.highlevel.model.Slice;
import be.iminds.ilabt.jfed.log.Logger;
import be.iminds.ilabt.jfed.log.ResultListener;
import be.iminds.ilabt.jfed.lowlevel.connection.JFedException;
import be.iminds.ilabt.jfed.lowlevel.api_wrapper.UserAndSliceApiWrapper;
import be.iminds.ilabt.jfed.util.common.GeniUrn;
import be.iminds.ilabt.jfed.highlevel.util.JavaFXLogger;
import org.slf4j.LoggerFactory;

import javax.annotation.Nonnull;
import javax.annotation.Nullable;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collection;
import java.util.List;

/**
 * User: twalcari
 * Date: 6/17/2015
 * Time: 15:24
 */
public final class UnregisterSliversTask extends Task {
    private static final org.slf4j.Logger LOG = LoggerFactory.getLogger(UnregisterSliversTask.class);

    @Nonnull private final Slice slice;
    @Nonnull  private final Collection<GeniUrn> deletedSliverUrns;
    @Nullable private final Task extraTaskToDependOn;
    @Nonnull private final Server auth;
    @Nonnull private final ResultListener[] extraResultListeners;

    @Nonnull private final UserAndSliceApiWrapper userAndSliceApiWrapper;
    @Nonnull private final JavaFXLogger logger;
    @Nonnull private final HighLevelTaskFactory highLevelTaskFactory;

    public UnregisterSliversTask(@Nonnull Slice slice,
                                 @Nonnull Server auth,
                                 @Nonnull  Collection<GeniUrn> deletedSliverUrns,
                                 @Nullable Task extraTaskToDependOn,
                                 @Nonnull UserAndSliceApiWrapper userAndSliceApiWrapper,
                                 @Nonnull JavaFXLogger logger,
                                 @Nonnull HighLevelTaskFactory highLevelTaskFactory,
                                 @Nonnull ResultListener... extraResultListeners) {
        super("UnregisterSliversTask slivers deleted @ " + auth.getDefaultComponentManagerUrn());

        this.slice = slice;
        this.deletedSliverUrns = deletedSliverUrns;
        this.auth = auth;
        this.extraTaskToDependOn = extraTaskToDependOn;
        this.userAndSliceApiWrapper = userAndSliceApiWrapper;
        this.logger = logger;
        this.highLevelTaskFactory = highLevelTaskFactory;
        this.extraResultListeners = extraResultListeners;
    }

    @Override
    protected void doTask(@Nonnull TaskExecution taskExecution) throws JFedException, InterruptedException {
        ResultListener[] resultListeners = Arrays.copyOf(extraResultListeners, extraResultListeners.length + 1);
        resultListeners[extraResultListeners.length] = taskExecution;
        Logger logger = JavaFXLogger.wrap(this.logger, resultListeners);

        Collection<GeniUrn> sliverUrns = deletedSliverUrns;
        LOG.debug("UnregisterSliversTask has {} sliver Urns to unregister.", sliverUrns.size());
        if (!sliverUrns.isEmpty())
            userAndSliceApiWrapper.unregisterAggregatesForSlice(logger,
                    slice.getCredentials(), slice.getUrn(),
                    auth.getDefaultComponentManagerAsGeniUrn(), sliverUrns);
    }

    @Nonnull
    @Override
    public List<Task> initDependsOn() {
        List<Task> res = new ArrayList<>();
        if (extraTaskToDependOn != null)
            res.add(extraTaskToDependOn);
        res.add(highLevelTaskFactory.getSliceCredential(slice));
        return res;
    }
}
